#!/bin/bash
#
# Copyright:
# ----------------------------------------------------------------------------
# This confidential and proprietary software may be used only as authorized
# by a licensing agreement from ARM Limited.
#      (C) COPYRIGHT 2016-2018 ARM Limited, ALL RIGHTS RESERVED
# The entire notice above must be reproduced on all authorized copies and
# copies may only be made to the extent permitted by a licensing agreement
# from ARM Limited.
# ----------------------------------------------------------------------------
set -e

TIME_START=`date +%s`

# Switch to the build directory
cd "$(dirname "${BASH_SOURCE[0]}")"

# Read settings written by bootstrap.bash
BOOTSTRAP=".bob.bootstrap"
source "${BOOTSTRAP}"

# Switch to the working directory
cd "${WORKDIR}"

# Generate config.go from Mconfig
python "${BOB_DIR}/scripts/generate_config_json.py" --database "${SRCDIR}/Mconfig" --output "${BUILDDIR}/config.json" ${BOB_CONFIG_OPTS} "${BUILDDIR}/${CONFIGNAME}"

TIME_1=`date +%s`

# If enabled, the following environment variables optimize the performance
# of ccache. Otherwise they have no effect.
# To build with ccache, set the environment variable CCACHE_DIR to where the
# cache is to reside and add CCACHE=y to the build config to enable.
export CCACHE_BASEDIR="$(readlink -f "${SRCDIR}")"
export CCACHE_CPP2=y
export CCACHE_SLOPPINESS=file_macro,time_macros

# Build the builder if necessary
BUILDDIR="${BUILDDIR}" SKIP_NINJA=true ${BOB_DIR}/blueprint/blueprint.bash
TIME_2=`date +%s`

# Do the actual build
ninja -f "${BUILDDIR}/build.ninja" -w dupbuild=err "$@"

# Send build times to server
TIME_END=`date +%s`
if [ -f "${BOB_DIR}/scripts/blueprint_build_stats.py" ]; then
	python "${BOB_DIR}/scripts/blueprint_build_stats.py" -ts "$TIME_START" -t1 "$TIME_1" -t2 "$TIME_2" -te "$TIME_END" -- "$@"
fi
