/*
 * Copyright:
 * ----------------------------------------------------------------------------
 * This confidential and proprietary software may be used only as authorized
 * by a licensing agreement from ARM Limited.
 *      (C) COPYRIGHT 2017-2018 ARM Limited, ALL RIGHTS RESERVED
 * The entire notice above must be reproduced on all authorized copies and
 * copies may only be made to the extent permitted by a licensing agreement
 * from ARM Limited.
 * ----------------------------------------------------------------------------
 */

package core

import (
	"reflect"

	"github.com/google/blueprint"
)

var aliasTag = DependencyTag{name: "alias"}

type Aliasable interface {
	getAliasList() []string
}

// Modules that can be aliased have these Blueprint properties
type AliasableProps struct {
	// Adds this module to an alias
	Add_to_alias []string
}

func (p *AliasableProps) getAliasList() []string {
	return p.Add_to_alias
}

// Properties supported by the bob_alias module
type AliasProps struct {
	// Modules that this alias will cause to build
	Srcs []string
	AliasableProps
}

// Type representing each bob_alias module
type Alias struct {
	blueprint.SimpleName
	Properties struct {
		AliasProps
		Features
	}
}

func (m *Alias) Features() *Features {
	return &m.Properties.Features
}

func (m *Alias) topLevelProperties() []interface{} {
	return []interface{}{&m.Properties.AliasProps}
}

func (m *Alias) getAliasList() []string {
	return m.Properties.getAliasList()
}

// Called by Blueprint to generate the rules associated with the alias.
// This is forwarded to the backend to handle.
func (m *Alias) GenerateBuildActions(ctx blueprint.ModuleContext) {
	getBackend(ctx).AliasActions(m, ctx)
}

// Create the structure representing the bob_alias
func aliasFactory(config *Config) (blueprint.Module, []interface{}) {
	module := &Alias{}
	module.Properties.Features.Init(config, reflect.TypeOf(AliasProps{}))
	return module, []interface{}{&module.Properties, &module.SimpleName.Properties}
}

// Setup dependencies between aliases and their targets
func aliasMutator(mctx blueprint.BottomUpMutatorContext) {
	if a, ok := mctx.Module().(*Alias); ok {
		parseAndAddVariationDeps(mctx, aliasTag, a.Properties.Srcs...)
	}
	if a, ok := mctx.Module().(Aliasable); ok {
		for _, s := range a.getAliasList() {
			mctx.AddReverseDependency(mctx.Module(), aliasTag, s)
		}
	}
}
