#!/usr/bin/env bash
#
# Copyright:
# ----------------------------------------------------------------------------
# This confidential and proprietary software may be used only as authorized
# by a licensing agreement from ARM Limited.
#      (C) COPYRIGHT 2016-2017 ARM Limited, ALL RIGHTS RESERVED
# The entire notice above must be reproduced on all authorized copies and
# copies may only be made to the extent permitted by a licensing agreement
# from ARM Limited.
# ----------------------------------------------------------------------------

# This script sets up the environment to be able to build the driver
# using Android paths only.

# This script should be called like:
# $PATH_TO_DDK/bldsys/generate_android_inc.bash $PATH_TO_DDK $BUILDDIR $GOROOT

# This script is invoked by Android.mk in a $(shell) expression, so its
# standard output is buffered. Swap stdout and stderr so that the output of
# this is visible.
exec 3>&2 2>&1 1>&3-

set -e

PATH_TO_DDK="$1"
BUILDDIR=$(readlink -f "$2")
GOROOT="$3"

if [ -x "${BUILDDIR}/buildme" -a -f "${BUILDDIR}/mali.config" ] ; then
    cd "${PATH_TO_DDK}"

    # The following would setup to use Androids prebuilt go. We don't
    # do this because we depend on Go 1.8 but still run on Android N
    # (which has an older version).
    #export GOPATH="${ANDROID_BUILD_TOP}"
    #export GOROOT="${GOPATH}/prebuilts/go/linux-x86"
    #export PATH="${GOROOT}/bin:${PATH}"

    # Use GOROOT recorded during setup_android call
    export GOROOT

    "$BUILDDIR/buildme"

    # To integrate with Kati, it is important that we output a different
    # value in the shell whenever the generated Andoid makefiles (*.inc)
    # change.
    #
    # To do that, call md5sum is output after the build of the Android file
    # has finished. This script would normally not be called manually, but
    # rather through Android.mk
    md5sum "$BUILDDIR"/*.inc | md5sum - >&2
else
    echo "${BUILDDIR}/buildme and ${BUILDDIR}/mali.config don't exist!"
    exit 1
fi

echo "Success" >&2
