# This confidential and proprietary software may be used only as
# authorised by a licensing agreement from ARM Limited
# (C) COPYRIGHT 2010-2011, 2013-2017 ARM Limited
# ALL RIGHTS RESERVED
# The entire notice above must be reproduced on all authorised
# copies and copies may only be made to the extent permitted
# by a licensing agreement from ARM Limited.

import os, re
import gpuinfo

# This dictionary contains an entry per platform configuration.
# Its keys are the platform configuration names selectable with the opencl_platform_config= option when building the
# driver and its values are the list platforms and the devices they contain.
# configuration.
OpenCLPlatformConfigs = {
       'config_t6xx':          [ 't6xx', 't6xx_cg1' ],
       'config_bfst':          [ 'bfst' ],
   }

def opencl_get_available_platform_configs( ):
	"""
	Return the list of available OpenCL platform configs
	"""
	configs = { "config_t6xx", "config_bfst" }

	return configs

def opencl_platform_config_get_default( gpu ):
	"""
	Return the CL configuration to use given a GPU product ID.
	"""
	def_cfg = None
	platforms = opencl_get_available_platform_configs()

	if not gpuinfo.is_bifrost(gpu):
		def_cfg = 'config_t6xx'
        else:
		def_cfg = 'config_bfst'

	if not def_cfg:
		raise Exception("bldsys error: GPU name '%s' is not associated to a family.\n" % (gpu))

	if not def_cfg in platforms:
		raise Exception("bldsys error: Could not find a suitable OpenCL platform for given GPU ID '{0}'".format( gpu ) )
	
	return def_cfg

def __opencl_platform_config_get_devices( config ):
	"""
	Private function that returns the complete list of devices supported by the selected configuration (including devices
	which are on different platforms)
	"""
	return OpenCLPlatformConfigs[ config ]

def opencl_platform_config_has_device( env, device ):
	"""
	Returns True if 'device' is used by any platform of the selected configuration and False otherwise
	"""
	devices = __opencl_platform_config_get_devices( env['opencl_platform_config'] )
	return device in devices

def opencl_platform_config_get_names():
	"""
	Returns the names of the supported platform configurations
	"""
	return OpenCLPlatformConfigs.keys()

